package com.sun.xml.internal.ws.client.sei;

import com.sun.xml.internal.ws.api.message.Headers;
import com.sun.xml.internal.ws.api.message.Header;
import com.sun.istack.internal.NotNull;
import javax.xml.namespace.QName;
import com.sun.xml.internal.ws.api.pipe.Fiber;
import com.sun.xml.internal.ws.client.AsyncResponseImpl;
import com.sun.xml.internal.ws.client.ResponseContextReceiver;
import com.sun.xml.internal.ws.client.RequestContext;
import com.sun.xml.internal.ws.api.message.Packet;
import java.lang.reflect.Modifier;
import java.lang.reflect.Proxy;
import com.sun.xml.internal.ws.api.server.Container;
import java.lang.reflect.InvocationTargetException;
import com.sun.xml.internal.ws.api.server.ContainerResolver;
import com.sun.istack.internal.Nullable;
import com.sun.xml.internal.ws.wsdl.OperationDispatcher;
import java.util.Iterator;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLBoundOperation;
import com.sun.xml.internal.ws.api.model.MEP;
import com.sun.xml.internal.ws.model.JavaMethodImpl;
import com.sun.xml.internal.ws.api.client.WSPortInfo;
import java.util.HashMap;
import com.sun.xml.internal.ws.api.addressing.WSEndpointReference;
import com.sun.xml.internal.ws.api.pipe.Tube;
import com.sun.xml.internal.ws.binding.BindingImpl;
import com.sun.xml.internal.ws.client.WSServiceDelegate;
import java.lang.reflect.Method;
import java.util.Map;
import com.sun.xml.internal.ws.api.SOAPVersion;
import com.sun.xml.internal.ws.model.SOAPSEIModel;
import com.sun.xml.internal.ws.api.databinding.Databinding;
import java.lang.reflect.InvocationHandler;
import com.sun.xml.internal.ws.client.Stub;

public final class SEIStub extends Stub implements InvocationHandler
{
    Databinding databinding;
    public final SOAPSEIModel seiModel;
    public final SOAPVersion soapVersion;
    private final Map<Method, MethodHandler> methodHandlers;
    
    @Deprecated
    public SEIStub(WSServiceDelegate owner, BindingImpl binding, SOAPSEIModel seiModel, Tube master, WSEndpointReference epr) {
        super(owner, master, binding, seiModel.getPort(), seiModel.getPort().getAddress(), epr);
        this.methodHandlers = new HashMap();
        this.seiModel = seiModel;
        this.soapVersion = binding.getSOAPVersion();
        this.databinding = seiModel.getDatabinding();
        this.initMethodHandlers();
    }
    
    public SEIStub(WSPortInfo portInfo, BindingImpl binding, SOAPSEIModel seiModel, WSEndpointReference epr) {
        super(portInfo, binding, seiModel.getPort().getAddress(), epr);
        this.methodHandlers = new HashMap();
        this.seiModel = seiModel;
        this.soapVersion = binding.getSOAPVersion();
        this.databinding = seiModel.getDatabinding();
        this.initMethodHandlers();
    }
    
    private void initMethodHandlers() {
        Map<WSDLBoundOperation, JavaMethodImpl> syncs = new HashMap();
        for (JavaMethodImpl m : this.seiModel.getJavaMethods()) {
            if (!m.getMEP().isAsync) {
                SyncMethodHandler handler = new SyncMethodHandler(this, m);
                syncs.put(m.getOperation(), m);
                this.methodHandlers.put(m.getMethod(), handler);
            }
            else {
                continue;
            }
        }
        for (JavaMethodImpl jm : this.seiModel.getJavaMethods()) {
            JavaMethodImpl sync = (JavaMethodImpl)syncs.get(jm.getOperation());
            if (jm.getMEP() == MEP.ASYNC_CALLBACK) {
                Method i = jm.getMethod();
                CallbackMethodHandler handler2 = new CallbackMethodHandler(this, i, i.getParameterTypes().length - 1);
                this.methodHandlers.put(i, handler2);
            }
            if (jm.getMEP() == MEP.ASYNC_POLL) {
                Method i = jm.getMethod();
                PollingMethodHandler handler3 = new PollingMethodHandler(this, i);
                this.methodHandlers.put(i, handler3);
            }
            else {
                continue;
            }
        }
    }
    
    @Nullable
    public OperationDispatcher getOperationDispatcher() {
        if (this.operationDispatcher == null && this.wsdlPort != null) {
            this.operationDispatcher = new OperationDispatcher(this.wsdlPort, this.binding, this.seiModel);
        }
        return this.operationDispatcher;
    }
    
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        this.validateInputs(proxy, method);
        Container old = ContainerResolver.getDefault().enterContainer(this.owner.getContainer());
        try {
            MethodHandler handler = (MethodHandler)this.methodHandlers.get(method);
            if (handler != null) {
                return handler.invoke(proxy, args);
            }
            try {
                return method.invoke(this, args);
            }
            catch (IllegalAccessException e) {
                throw new AssertionError(e);
            }
            catch (IllegalArgumentException e2) {
                throw new AssertionError(e2);
            }
            catch (InvocationTargetException e3) {
                throw e3.getCause();
            }
        }
        finally {
            ContainerResolver.getDefault().exitContainer(old);
        }
    }
    
    private void validateInputs(Object proxy, Method method) {
        if (proxy == null || !Proxy.isProxyClass(proxy.getClass())) {
            throw new IllegalStateException("Passed object is not proxy!");
        }
        Class<?> declaringClass = method.getDeclaringClass();
        if (method == null || declaringClass == null || Modifier.isStatic(method.getModifiers())) {
            throw new IllegalStateException("Invoking static method is not allowed!");
        }
        return;
    }
    
    public final Packet doProcess(Packet request, RequestContext rc, ResponseContextReceiver receiver) {
        return super.process(request, rc, receiver);
    }
    
    public final void doProcessAsync(AsyncResponseImpl<?> receiver, Packet request, RequestContext rc, Fiber.CompletionCallback callback) {
        super.processAsync(receiver, request, rc, callback);
    }
    
    @NotNull
    protected final QName getPortName() {
        return this.wsdlPort.getName();
    }
    
    public void setOutboundHeaders(Object... headers) {
        if (headers == null) {
            throw new IllegalArgumentException();
        }
        Header[] hl = new Header[headers.length];
        int i = 0;
        while (i < hl.length) {
            if (headers[i] == null) {
                throw new IllegalArgumentException();
            }
            hl[i] = Headers.create(this.seiModel.getBindingContext(), headers[i]);
            ++i;
        }
        super.setOutboundHeaders(hl);
        return;
    }
}
